"""
This is my first try at creating an NFG with PyMC objects. 
c1 ~ Uniform(0, 1)
    c1's space is [0, 1], and the pdf is uniform(0, 1)
c2 ~ Bernoulli(p=c1)
    c2's space is {0, 1}, and the cpd is Bernoulli(p=c1)
d1 ~ CPT(c2)
    d1's space is {0, 1}, and the cpd is given by a user-defined CPT on c2
d2 ~ CPT(d1,y)
    d2's space is {0, 1}, and the cpd is given by a user-defined CPT on (d1, y)
c3 ~ Deterministic(c1, d1, d2)
    The value is: 
        P1 if |d1-c1|<|d2-c1|, or 
        P2 if |d1-c1|>|d2-c1|, or
        Tie if |d1-c1|=|d2-c1|.
R = {d1, d2, {c1, c2, c3}}
U1 = 1 if c3=P1, 0 if c3=P2, 1/2 if c3=Tie
U2 = 1-U1
""" 

from pymc import *
import numpy as np

#Define Chance Nodes and CPD's in NFG-speak
c1 = Uniform('c1', 0, 1) 
c2 = Bernoulli('c2', p=c1)

#Define space of chance node 1 (Chance node 2 is continuous 0 to 1)
c2_space = np.array([0,1])

#Define Deterministic Policy of Player 1
@deterministic(plot=False)
def d1(x=c2):
    ''' p1 has a dominant strategy '''
    out = c2
    return out

#Define space of player 1's actions
d1_space = np.array([0,1])

#Define Stochastic Policy of Player 2
@pymc.stochastic(dtype=int)
def d2(value=0, c2=0, d1=0):
    """p2 has a stochastic strategy supplied by a solution concept"""

    def logp(value, c2, d1):
        '''Log probs will be computed from solution_concept'''

    def random(t_l, t_h):
        '''random draws will be taken by policy_draw'''

#Define space of player 2's actions
d2_space = d1_space