function m = Attacker_L0( Attacker, MA, params )
%Simulate the attacker's policy
%Inputs:    Attacker = attacker
%           MA = attacker's memory
%Output:    mnew = Attacker's move
%global params;

%Choose the policy to play according to params.L0_mode
%Parameters to the policy are generally in Attacker.thetaA

if Attacker.exist == 0 % does not exist
    m = 0;
    
else % exist
    %Fixed move
    if( Attacker.mode == 1 )
        m = Attacker.thetaA;
        assert( m >= -params.M && m <= params.M );
    %Randomize uniformly between {-M,...,0,...,M}
    elseif( Attacker.mode == 2 )
        m = randi([-params.M,params.M],1);
    %Play according to Sec 3.5 in writeup
    elseif( Attacker.mode == 3 )
        V2 = MA.V2;
        [mstar,Vstar] = Find_star(V2, MA, params);
    
        if Vstar > Attacker.thetaA
            m = mstar;
        else
            m = MA.m - sign(V2-params.V0);
        end
    end
end

%Saturator
m = min(m, params.M);
m = max(m, -params.M);

end

function [mstar,Vstar] = Find_star( V2, MA, params )
%Find the star quantities as in Scott's writeup
%global params;

q3max = params.p3max;

m_prev = MA.m;
m = [-params.M:params.M]';

arg = abs(V2+(m-m_prev)*q3max*params.x2/params.M/params.V0-params.V0);
[~,ind]=max(arg);
mstar = m(ind);
Vstar = abs(max(arg));

end