function [defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
    Av_defAvgReward, Av_defAvgRewardSEM, Av_attAvgReward, Av_attAvgRewardSEM] ...
    = Average_rewards_d( Defender, Attacker, N, parallel, params,...
                                   BinsOrSet, CPTs, ER )
% Compute the players' average rewards over N episodes
% Inputs:    Defender = defender
%            Attacker = attacker
%            N = number of episodes to average
%            parallel = set to 'true' to use parallel computing toolbox
%            global parameters
%            BinsOrSet
%            CPTs

% Make sure number of samples is positive
assert( N > 0 );

meanRD = zeros(1,N);
meanRA = zeros(1,N);

Av_meanRD = zeros(N,params.nSimSteps); % Average of the mean defender rewards
AV_meanRA = zeros(N,params.nSimSteps); % Average of the mean attacker rewards

tic     %start timer
for i=1:N
    i
    
    Attacker.exist = genAttExists(params.ProbAttExist);
    %calculate the rewards for each episode
    [~, defRewards, attRewards, Av_defRewards, Av_attRewards] = Simulate_d( Defender, Attacker, false,params,...
                                            BinsOrSet, CPTs, ER );
    %total rewards over each episode
    meanRD(i) = sum(defRewards);
    meanRA(i) = sum(attRewards);
    
    Av_meanRD(i,:)=Av_defRewards;
    Av_meanRA(i,:)=Av_attRewards;
    
end
toc     %stop timer

%average over N episodes
defAvgReward = mean(meanRD);
defAvgRewardSEM = sqrt(var(meanRD)/N);
attAvgReward = mean(meanRA);
attAvgRewardSEM = sqrt(var(meanRA)/N);


Av_defAvgReward = mean(Av_meanRD);
Av_defAvgRewardSEM = sqrt(var(Av_meanRD)/N);

Av_attAvgReward = mean(Av_meanRA);
Av_attAvgRewardSEM = sqrt(var(Av_meanRA)/N);

figure(121)
errorbar(Av_defAvgReward,Av_defAvgRewardSEM,'k','LineWidth',2);
title('Defender average rewards averaged over 50 simulations');
xlabel('Simulation steps','fontsize',16,'fontweight','b');
grid


figure(122)
errorbar(Av_attAvgReward,Av_attAvgRewardSEM,'r','LineWidth',2);
title('Attacker average rewards averaged over 50 simulations');
xlabel('Simulation steps','fontsize',16,'fontweight','b');
grid

end