function [CPT_V2] = GenCPT_V2(params, BinsOrSet,T)
% This function is used to generate the conditional probability table for
% V2
% Input: Global parameters
%        the set of V1 and the bins of L and A
% Output: CPT_V2
    
    % Initialize CPT_V2. V2 = V2(V1,L,A)
    % q2 = a*p2. p2 fully determines L
    % p3 is fixed. q3 fully determines A
    CPT_V2 = zeros(BinsOrSet.N_V2, BinsOrSet.N_V1, BinsOrSet.N_p2,...
        BinsOrSet.N_q3);
    
    if ( ~exist('T') )
        T = 10000; % Number of times we repeat for each combination of inputs
    end
    
    for k = 1:BinsOrSet.N_q3 % for each bin of q3
        q3 = BinsOrSet.q3Set(k) * ones(T,1);
        for j = 1:BinsOrSet.N_p2 % for each bin of p2
            p2 = BinsOrSet.p2bins(j) + (BinsOrSet.p2bins(j+1) - ...
                BinsOrSet.p2bins(j)) .* rand(T,1); % Generate p2 uniformly from the bin
            q2 = params.a * p2; % The aggregate load at node 2 has a constant power factor
            for i = 1:BinsOrSet.N_V1 % for each bin of V1
                V1 = BinsOrSet.V1set(i) * ones(T,1); % Repeat T times
                
                Num = zeros(BinsOrSet.N_V2,1); % Initialize the tracker
                
                V2 = V1-(params.r1*(-params.p3+p2) + params.x1*(-q3+q2))/params.V0; % Update V2 given inputs
                [~,p] = histc(V2,BinsOrSet.V2n3bins); % Map V2 back to an bin
                for s = 1:BinsOrSet.N_V2 % for each bin of V2
                    Num(s,1) = sum(p==s); % Count the number that this bin is arrived
                end
                CPT_V2(:,i,j,k) = Num(:,1)/T; % Probabilities given V1, L, A
            end
        end
    end
end
                        
                    
            
            
            
            
            
            