function [ER_Att] = Expected_AttReward(params, BinsOrSet,T)
% This function is used to compute the expected reward of attacker
% in each combination of bins for V2. R_A = R_A(V2)
% Input: global parameters
%        bins of V2
% Output: ER_Att

if ( ~exist('T') )
    T = 10; % Number of times we repeat for each bin of V2
end

ER_Att = zeros(BinsOrSet.N_V2, 1); % Initialize the output

for i = 1:BinsOrSet.N_V2 % for each bin of the V2
    % Generate V2 uniformly from the bin
    if (i == 1) % Apply half-Gussian distribution. Only sample the left to the mean
        mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
        sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
        %             Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
        Ran = mu + sigma.*randn(T,1);
        V2 = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
    elseif (i == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean.
        mu = BinsOrSet.V2n3bins(i); % mean = lowerbound of the highest bin
        sigma = (BinsOrSet.V2n3bins(i) - BinsOrSet.V2n3bins(i-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
        %             Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
        Ran = mu + sigma.*randn(T,1);
        V2 = BinsOrSet.V2n3bins(i) + abs(BinsOrSet.V2n3bins(i) - Ran); % Convert full-normal to half-normal
    else % Sample uniformly in the medium three bins
        %             V2 = random('Uniform', BinsOrSet.V2n3bins(i),BinsOrSet.V2n3bins(i+1), T, 1);
        V2 = BinsOrSet.V2n3bins(i) + (BinsOrSet.V2n3bins(i+1)-BinsOrSet.V2n3bins(i)).*rand(T,1);
    end
    
    
    if params.Att_gradualReward == 1
        R_A = Heaviside( V2/params.V0-(1+0.06) )*10 ...
            + Heaviside( (1-0.06)-V2/params.V0 )*10 ...
            + (Heaviside(V2/params.V0-(1+0.04)) - Heaviside(V2/params.V0-(1+0.06)) ...
            + Heaviside((1-0.04)-V2/params.V0) - Heaviside((1-0.06)-V2/params.V0))*2 ...
            + (Heaviside(V2/params.V0-(1+0.02)) - Heaviside(V2/params.V0-(1+0.04)) ...
            + Heaviside((1-0.02)-V2/params.V0) - Heaviside((1-0.04)-V2/params.V0))*1;
        
        %this uses 1.05 and 0.95 as the scoring thresholds.
%         R_A = Heaviside( V2/params.V0-(1+params.eps) )*10 ...
%             + Heaviside( (1-params.eps)-V2/params.V0 )*10 ...
%             + (Heaviside(V2/params.V0-(1+0.03)) - Heaviside(V2/params.V0-(1+params.eps)) ...
%             + Heaviside((1-0.03)-V2/params.V0) - Heaviside((1-params.eps)-V2/params.V0))*3;
    else
        
        R_A = Heaviside( V2/params.V0-(1+params.eps) ) ...
            + Heaviside( (1-params.eps)-V2/params.V0 );
    end
    
    ER_Att(i,1) = sum(R_A)/T; % Expected reward
end
end


function H = Heaviside( arg )
H = (arg >= 0 );
end


