function [ER_Def] = Expected_DefReward(params, BinsOrSet,T)
% This function is used to compute the expected reward of defender in
% in each combination of bins for V2 and V3. R_D = R_D(V2,V3)
% Input: global parameters
%        Bins for V2 and V3
% Output: ER_Def

    if ( ~exist('T') )
        T = 10; % Number of times to repeat for each combination of bins for V2 and V3
    end
    
    ER_Def = zeros(BinsOrSet.N_V2, BinsOrSet.N_V3); % Initialize the output
    
    for j = 1:BinsOrSet.N_V3 % for each bin of V3
        % Generate V uniformly from the bin
        if (j == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
            mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
            sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
%             Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
            Ran = mu + sigma.*randn(T,1);
            V3 = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
        elseif (j == BinsOrSet.N_V3) % Apply half-Guassian distribution. Only sample the right to the mean. 
            mu = BinsOrSet.V2n3bins(j); % mean = lowerbound of the highest bin
            sigma = (BinsOrSet.V2n3bins(j) - BinsOrSet.V2n3bins(j-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
%             Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
            Ran = mu + sigma.*randn(T,1);
            V3 = BinsOrSet.V2n3bins(j) + abs(BinsOrSet.V2n3bins(j) - Ran); % Convert full-normal to half-normal
        else % Sample uniformly in the medium three bins
%             V3 = random('Uniform', BinsOrSet.V2n3bins(j),BinsOrSet.V2n3bins(j+1), T, 1);
            V3 = BinsOrSet.V2n3bins(j) + (BinsOrSet.V2n3bins(j+1)-BinsOrSet.V2n3bins(j)).*rand(T,1);
        end
        
        for i = 1:BinsOrSet.N_V2 % for each bin of V2
            % Generate V2 uniformly from the bin
            if (i == 1) % Apply half-Gussian distribution. Only sample the left to the mean 
                mu = BinsOrSet.V2n3bins(2); % mean = upperbound of the lowest bin
                sigma = (BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/sqrt(1-2/pi); % Stdev = range of right neighboring bin divided by a difference factor
%                 Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
                Ran = mu + sigma.*randn(T,1);
                V2 = BinsOrSet.V2n3bins(2) - abs(BinsOrSet.V2n3bins(2) - Ran); % Convert full-normal to half-normal
            elseif (i == BinsOrSet.N_V2) % Apply half-Guassian distribution. Only sample the right to the mean. 
                mu = BinsOrSet.V2n3bins(i); % mean = lowerbound of the highest bin
                sigma = (BinsOrSet.V2n3bins(i) - BinsOrSet.V2n3bins(i-1))/sqrt(1-2/pi); % Stdev = range of the left neighboring bin divided by a difference factor
%                 Ran = random('Normal', mu, sigma, T, 1); % Generate full-normally distributed numbers
                Ran = mu + sigma.*randn(T,1);
                V2 = BinsOrSet.V2n3bins(i) + abs(BinsOrSet.V2n3bins(i) - Ran); % Convert full-normal to half-normal
            else % Sample uniformly in the medium three bins
%                 V2 = random('Uniform', BinsOrSet.V2n3bins(i),BinsOrSet.V2n3bins(i+1), T, 1);
                V2 = BinsOrSet.V2n3bins(i) + (BinsOrSet.V2n3bins(i+1)-BinsOrSet.V2n3bins(i)).*rand(T,1);
            end 
            
            % Defender's reward function
            R_D = -((V2/params.V0 - 1)/params.eps).^2 - ((V3/params.V0 - 1)/params.eps).^2;
%             R_D = R_D + 30; % 30 is the cap
            
            ER_Def(i,j) = sum(R_D)/T; % Expected reward
        end
    end
end
            
            