function [MAnew] = Sample_MA_d(MAprev, OAnew, Aindex_prev, CPTs, BinsOrSet, params)
% Sample attacker's memory @ t
% Inputs: MA@t-1, OA@t, A@t-1, global parameters
% Output: MAnew


MAnew.V2index = OAnew.V2index;          % V2_@ t
MAnew.V3index = OAnew.V3index;          % V3_@ t
MAnew.V3index_prev = MAprev.V3index;    % V3_@ t-1
MAnew.V3index_2prev = MAprev.V3index_prev; % V3_@ t-2
MAnew.q3index_prev = Aindex_prev.q3index; %AA @ t-1
MAnew.q3index_2prev = MAprev.q3index_prev; %AA @ t-2

% Compute the statistic of attacker @ t
MAnew.SAindex = Weighted_sample(CPTs.statisticA(:,MAprev.SAindex,...
    MAnew.V3index_prev, MAnew.V3index_2prev, ...
    MAnew.q3index_prev, MAnew.q3index_2prev));

MAnew.EA_prev = MAprev.EA;  % Remember previous region

%$$ Figure out what region we're currently in
% first figure out the mean of the current V2 bin
if OAnew.V2index ==1
    V2 = BinsOrSet.V2n3bins(2)-(BinsOrSet.V2n3bins(3)-BinsOrSet.V2n3bins(2))/2;
elseif OAnew.V2index == BinsOrSet.N_V2
    V2 = BinsOrSet.V2n3bins(OAnew.V2index) + (BinsOrSet.V2n3bins(OAnew.V2index) ...
        - BinsOrSet.V2n3bins(OAnew.V2index-1))/2;
else
    V2 = BinsOrSet.V2n3bins(OAnew.V2index) + (BinsOrSet.V2n3bins(OAnew.V2index+1) ...
        - BinsOrSet.V2n3bins(OAnew.V2index))/2;
end


mPrev = Aindex_prev.q3index-6;
[mStar, VStar] = Find_star(V2, mPrev, params, BinsOrSet); % Find stars

thetaA=0.07;
if VStar > thetaA
    MAnew.EA = 1;
else
    MAnew.EA = 2;
end

end

function [mStar, VStar] = Find_star(V2,mPrev,params, BinsOrSet)
% This function is used to find the star quantities as in Scott's write-up
% Inputs: V2; previous m; global parameters
% Outputs: mStar; VStar

q3max = params.p3max; % q3 can vary from -p3max to p3max

% Replicate the arrays for the purpose of vectorization
% Rep_mSet = repmat(BinsOrSet.mSet, T, 1);
Rep_V2 = repmat(V2, 1, BinsOrSet.N_m);

% According to Section 3.5 in Scott's writeup
arg = abs(Rep_V2+(BinsOrSet.mSet-mPrev)*q3max*params.x2/params.M/params.V0-params.V0);
[~,ind]= max(arg,[],2); % the index of the largest value in every row
mStar = (BinsOrSet.mSet(ind))';
VStar = abs(max(arg,[],2));


end