function S = Sample_S_d( S_prev, D_prev, A_prev, CPTs, params, BinsOrSet )
    S.L = Sample_L(params, BinsOrSet); % L = [p2index,q2]'
    S.V1index = Sample_V1(S_prev.V1index, D_prev, CPTs); % V1 = V1(V1@t-1, dX@t-1)
    S.V2index = Sample_V2(S.V1index, S.L, A_prev, CPTs); % V2 = V2(V1,L,A@t-1)
    S.V3index = Sample_V3(S.V2index, A_prev, CPTs); % V3 = V3(V2, A@t-1)
end
    
function [V1new] = Sample_V1(V1index_prev, D_prev, CPTs)
% Sample V1 
% Inputs: the index of V1@t-1 in the set of V1
%         Defender's action index D@t-1
%         CPT_V1
%         global parameters
% Output: V1index@t, i.e. the index of the new V1 in the set of V1
    
    % Do a weighted sample to determine V1
    V1new = Weighted_sample(CPTs.V1(:,V1index_prev, D_prev));
    
end

function [V2new] = Sample_V2(V1index, L, A_prev, CPTs)
% Sample V2
% Inputs: the bins of V1@t, L@t and A@t-1 and CPT_V2
% Output: V2index@t, i.e. the bin of the new V2

    p2index = L(1,1); % the bin of p2. L = [p2index,q2]'
    q3index = A_prev.q3index; % the bin of q3. A = [p3,q3index]', where p3 is fixed
    
    % Do a weighted sample to determine V2index
    V2new = Weighted_sample(CPTs.V2(:, V1index, p2index, q3index));
    
end

function [V3new] = Sample_V3(V2index, A_prev, CPTs)
% Sample V3
% Inputs: the bin of V2@t, A@t-1, CPT_V3
% Output: V3index@t, i.e. the bin of the new V3

    q3index = A_prev.q3index; % the bin of q3. A = [p3,q3index]', where p3 is fixed
    
    % Do a weighted sample to determine V3index
    V3new = Weighted_sample(CPTs.V3(:, V2index, q3index));
    
end

function [Lnew] = Sample_L(params,BinsOrSet)
% Sample L
% Input: Global parameters
%        bins of p2
% Output: L = [p2index, q2]', where p2index is the bin of p2.

    p2index = randi(BinsOrSet.N_p2); % Uniformly pick a bin
    
    p2 = BinsOrSet.p2bins(p2index) + (BinsOrSet.p2bins(p2index+1) - ...
        BinsOrSet.p2bins(p2index)) * rand(1,1); % Uniformly generate a p2 in the bin
    q2 = params.a * p2; % The aggregate load at node 2 has a constant power factor
    Lnew = [p2index, q2]';
    
end