function [states, defRewards, attRewards, Av_defRewards, Av_attRewards ] = Simulate_d(Defender,...
    Attacker, makePlot, params, BinsOrSet, CPTs, ER)
% Simulate an episode in a discrete environment
% Inputs: Defender = defender
%         Attacker = attacker
%         makePlot = set to 'true' to display plots
%         global params
%         BinsOrSet
%         CPTs
% Output: states = [V1,V2,V3,dX,A,L]'
%         defender's rewards; attacker's rewards

    fprintf('\nDiscrete environment simulation');
    if ( params.player_type == 1 )
        assert(false);
    elseif ( params.player_type == 2 )
        fprintf('\nDiscrete players');
    end

    % Initialize current time step as empty
    B = [];
    
    % Initialize the output
    states = cell(1,params.nSimSteps);
    defRewards = zeros(1, params.nSimSteps);
    attRewards = zeros(1, params.nSimSteps);
     
    defrew_av=0;   % average reward for the defender
    attrew_av=0;   % average reward for the attacker
    Av_defRewards = zeros(1,params.nSimSteps);
    Av_attRewards = zeros(1,params.nSimSteps);

    for t = 1:params.nSimSteps
        
        B = Sample_slice_d(B, Defender, Attacker, CPTs, params, BinsOrSet); % Simulate forward one time step
        [S,MD,MA,D,A,L] = Slice_unpack_d(B);
        
        % Save to output
        states{t} = B;
        
        % Rewards
        defRewards(1,t) = ER.Def(S.V2index,S.V3index); % B{2} = V2, B{3} = V3
        attRewards(1,t) = ER.Att(S.V2index);
        
        defrew_av=defrew_av+(ER.Def(S.V2index,S.V3index)-defrew_av)/t;
        attrew_av=attrew_av+(ER.Att(S.V2index)-attrew_av)/t;
        
        Av_defRewards(t) = defrew_av;
        Av_attRewards(t) = attrew_av;
        
    end

    % Set flag to plot
    if ( makePlot )
        %Plot the various variables over time
        Plot_S_d(states, BinsOrSet);
        Plot_V1_d(states, BinsOrSet);
        Plot_V2_d(states, BinsOrSet);
        Plot_V3_d(states, BinsOrSet);
        Plot_MD_d(states);
        Plot_MA_d(states);
        Plot_Defender_d(states,BinsOrSet);
        Plot_Attacker_d(states,BinsOrSet);
        Plot_L_d(states,BinsOrSet);
        Plot_RD_d(defRewards,Av_defRewards);
        Plot_RA_d(attRewards,Av_attRewards);
    end

end