clear all;
close all;

%Call this first to be able to call functions from sub-directories
addpath(genpath(pwd));

%Must first call these scripts to set parameters and load matrices
ppack.params = Set_params();
ppack = Load_mats(ppack);

Attacker.mode = 3;
Attacker.thetaA = 0.07;

% Sample uniformly on V2
N = 10000;  % Number of samples
V2min = 0.9;
V2max = 1.1;
V2 = V2min + (V2max-V2min).*rand(N,1);
q3index_prev = randi(ppack.BinsOrSet.N_q3, N, 1);

q3_c = zeros(N,1);
q3_d = zeros(N,1);
for i = 1:N
    % Continuous
    MA_c.V2 = V2(i);
    MA_c.q3_prev = ppack.BinsOrSet.q3Set( q3index_prev(i) );

    % Discrete
    MA_d.V2index = Discretize_V2( V2(i), ppack.BinsOrSet );
    MA_d.q3index_prev = q3index_prev(i);
    
    A_c = Attacker_policy(Attacker, MA_c, ppack.params);
    q3_c(i) = A_c.q3;
    A_d = Attacker_policy_d(Attacker, MA_d, ppack.CPTs, ppack.params, ppack.BinsOrSet);
    q3_d(i) = A_d.q3index;
end

figure;
subplot(3,1,1);
plot(q3_c(1:100), '-x');
title('Continuous');
subplot(3,1,2);
plot(q3_d(1:100), '-x');
title('Discrete');
subplot(3,1,3);
[AX,H1,H2]=plotyy(1:100,q3_c(1:100), 1:100,q3_d(1:100));
set(AX(1), 'YLim', [-ppack.params.p3max ppack.params.p3max]);
set(AX(2), 'YLim', [1 11]);

figure;
subplot(1,2,1);
hist(q3_c, ppack.BinsOrSet.N_q3);
title('Continuous');
subplot(1,2,2);
hist(q3_d, ppack.BinsOrSet.N_q3);
title('Discrete');

[v2index,q3index_prev]=meshgrid(1:ppack.BinsOrSet.N_V2, 1:ppack.BinsOrSet.N_q3);
figure;
for sp=1:ppack.BinsOrSet.N_q3
    subplot(3,4,sp);
    surf(v2index,q3index_prev,squeeze(ppack.CPTs.ScottAttL0(sp,:,:))');
end