clear all;
close all;

%Call this first to be able to call functions from sub-directories
addpath(genpath(pwd));

%Must first call these scripts to set parameters and load matrices
ppack.params = Set_params();
ppack = Load_mats(ppack);

Defender.mode = 3;
Defender.thetaD = 0;

% Sample uniformly on V2
N = 10000;  % Number of samples
V1min = 0.9;
V1max = 1.1;
V2min = 0.9;
V2max = 1.1;
V3min = 0.9;
V3max = 1.1;
V1 = V1min + (V1max-V1min).*rand(N,1);
V2 = V2min + (V2max-V2min).*rand(N,1);
V3 = V3min + (V3max-V3min).*rand(N,1);

dX_c = zeros(N,1);
dX_d = zeros(N,1);
for i = 1:N
    % Continuous
    MD_c.V1 = V1(i);
    MD_c.V2 = V2(i);
    MD_c.V3 = V3(i);

    % Discrete
    MD_d.V1index = Discretize_V1( V1(i), ppack.BinsOrSet );
    MD_d.V2index = Discretize_V2( V2(i), ppack.BinsOrSet );
    MD_d.V3index = Discretize_V3( V3(i), ppack.BinsOrSet );
    
    dX_c(i) = Defender_policy(Defender, MD_c, ppack.params);
    dX_d(i) = Defender_policy_d(Defender, MD_d, ppack.CPTs, ppack.params, ppack.BinsOrSet);
    
end

figure;
subplot(3,1,1);
plot(dX_c(1:100), '-xb');
title('Continuous');
subplot(3,1,2);
plot(dX_d(1:100), '-xg');
title('Discrete');
subplot(3,1,3);
[AX,H1,H2]=plotyy(1:100,dX_c(1:100), 1:100,dX_d(1:100));
set(AX(1), 'YLim', [-1 1]);
set(AX(2), 'YLim', [1 3]);

figure;
subplot(1,2,1);
hist(dX_c, ppack.BinsOrSet.N_q3);
title('Continuous');
subplot(1,2,2);
hist(dX_d, ppack.BinsOrSet.N_q3);
title('Discrete');
