%$$ The following code uses Exhaustive search for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

%$ Select the parameters on which the optimization will run
%$$
starttime = tic;
p2max = linspace(1.40,2.60,31); %(1.40:0.04:2.60);
p3max = linspace(0.60,2.00,36); %(0.60:0.04:2.00);
[p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
[FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim;

%preallocation
tempz = zeros(FirstDim,SecondDim);
SW_track = tempz;
SW_SEM_track = tempz;
AAR_track = tempz;
AAR_SEM_track = tempz;
DAR_track = tempz;
DAR_SEM_track = tempz;
DAM_track = tempz;
DAM_SEM_track = tempz;

clear ppack;
clear params;
ppack.params = Set_params();
ppack = Load_mats(ppack);
ppack.params.player_type = 2; %1=continuous players; 2=discrete
ppack.useContL0 = 0;  %0=Discrete players; 1=continuous players

for i = 1:FirstDim
    for j = 1:SecondDim
        ppacks{i,j} = ppack;
        ppacks{i,j}.params.p2max = p2max_in(i,j);
        ppacks{i,j}.params.p3max = p3max_in(i,j);
    end
end

% matlabpool open;
for i=1:FirstDim
    SW_tracki = zeros(SecondDim, 1);
    SW_SEM_tracki = zeros(SecondDim, 1);
    AAR_tracki = zeros(SecondDim, 1);
    AAR_SEM_tracki = zeros(SecondDim, 1);
    DAR_tracki = zeros(SecondDim, 1);
    DAR_SEM_tracki = zeros(SecondDim, 1);
    DAM_tracki = zeros(SecondDim, 1);
    DAM_SEM_tracki = zeros(SecondDim, 1);

    for j=1:SecondDim
        i
        j
        
        datetime = int2str(datevec(now));      %current date and time
        datetime(ismember(datetime,' ')) = []; %remove spaces
        fnamee=[datetime '_Exhaustive_p2max=' num2str(p2max_in(i,j)) '_p3max=' ...
                    num2str(p3max_in(i,j))];
                
        [filename, SW, SW_SEM, AAR, AAR_SEM, DAR, DAR_SEM,...
            DAM, DAM_SEM] = TrainD1JandSave_SW2a( ppacks{i,j}, fnamee);
        
        %for tracking
        SW_tracki(j) = SW;
        SW_SEM_tracki(j) = SW_SEM;
        AAR_tracki(j) = AAR;
        AAR_SEM_tracki(j) = AAR_SEM;
        DAR_tracki(j) = DAR;
        DAR_SEM_tracki(j) = DAR_SEM;
        DAM_tracki(j) = DAM;
        DAM_SEM_tracki(j) = DAM_SEM;
    end
    
    SW_track(i,:) = SW_tracki;
    SW_SEM_track(i,:) = SW_SEM_tracki;
    AAR_track(i,:) = AAR_tracki;
    AAR_SEM_track(i,:) = AAR_SEM_tracki;
    DAR_track(i,:) = DAR_tracki;
    DAR_SEM_track(i,:) = DAR_SEM_tracki;
    DAM_track(i,:) = DAM_tracki;
    DAM_SEM_track(i,:) = DAM_SEM_tracki;
end
% matlabpool close force local;

endtime = toc(starttime);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_ExhaustiveFinal'];
save(fnamee);

figure;
surf(p2max_in, p3max_in, SW_track);
xlabel('p2max');
ylabel('p3max');
zlabel('SW');

figure;
surf(p2max_in, p3max_in, DAR_track);
xlabel('p2max');
ylabel('p3max');
zlabel('defAvgReward');

figure;
surf(p2max_in, p3max_in, DAM_track);
xlabel('p2max');
ylabel('p3max');
zlabel('defAvgMoves');

fprintf('\nLoadAndRun Done!\n\n');



