%$$ The following code uses Exhaustive search for finding the optimum
%$$ values for the grid paramaters that will maximize a given social
%$$ welfare

clc
clear all
close all

%$ Select the parameters on which the optimization will run
%$$
starttime = tic;
p2max = linspace(1.40,2.60,31); %(1.40:0.04:2.60);
p3max = linspace(0.60,2.00,36); %(0.60:0.04:2.00);
[p2max_in,p3max_in] = ndgrid(p2max,p3max); %generate the grid
[FirstDim,SecondDim]=size(p2max_in);       %size of optimization space
NumPoint = FirstDim*SecondDim;

%preallocation
defAvgReward_track = zeros(FirstDim, SecondDim);
defAvgRewardSEM_track = zeros(FirstDim, SecondDim);
SW_track = zeros(FirstDim,SecondDim);
SW_SEM_track = zeros(FirstDim,SecondDim);
defAvgNumMove_track = zeros(FirstDim,SecondDim);

clear ppack;
clear params;
ppack.params = Set_params();
ppack.BinsOrSet = Save_BinsOrSet(ppack.params);
ppack.params.player_type = 1; %continuous players
temp = load ('Discrete/Load/ER.mat', 'ER');
ppack.ER = temp.ER;

for i = 1:FirstDim
    for j = 1:SecondDim
        ppacks{i,j} = ppack;
        ppacks{i,j}.params.p2max = p2max_in(i,j);
        ppacks{i,j}.params.p3max = p3max_in(i,j);
    end
end

matlabpool open;
parfor i=1:FirstDim
    defAvgReward_tracki = zeros(SecondDim, 1);
    defAvgRewardSEM_tracki = zeros(SecondDim, 1);
    SW_tracki = zeros(SecondDim, 1);
    SW_SEM_tracki = zeros(SecondDim, 1);
    defAvgNumMove_tracki = zeros(SecondDim, 1);
    for j=1:SecondDim
        i
        j
        
        [defAvgReward, defAvgRewardSEM,SW,SW_SEM,defAvgNumMove] = TrainD1JandSave_SW3(ppacks{i,j});
        
        %for tracking
        defAvgReward_tracki(j) = defAvgReward;
        defAvgRewardSEM_tracki(j) = defAvgRewardSEM;
        SW_tracki(j) = SW;
        SW_SEM_tracki(j) = SW_SEM;
        defAvgNumMove_tracki(j) = defAvgNumMove;
    end
    defAvgReward_track(i,:) = defAvgReward_tracki;
    defAvgRewardSEM_track(i,:) = defAvgRewardSEM_tracki;
    SW_track(i,:) = SW_tracki;
    SW_SEM_track(i,:) = SW_SEM_tracki;
    defAvgNumMove_track(i,:) = defAvgNumMove_tracki;
end
matlabpool close force local;

endtime = toc(starttime);

datetime = int2str(datevec(now));      %current date and time
datetime(ismember(datetime,' ')) = []; %remove spaces
fnamee=[datetime '_ExhaustiveSearch'];
save(fnamee);

figure;
surf(p2max_in, p3max_in, SW_track);
xlabel('p2max');
ylabel('p3max');
zlabel('SW');

figure;
surf(p2max_in, p3max_in, defAvgReward_track);
xlabel('p2max');
ylabel('p3max');
zlabel('defAvgReward');

fprintf('\nLoadAndRun Done!\n\n');



