function fileroot = SaveAllOpenPlots( fileroot )
%Saves all open figures to a pdf
%Inputs: fileroot = string for the filename (without extension)
%           if you leave this empty, it will use the current datetime
%Outputs: fileroot (same as input), in case you want to chain these
%           together

datetime = int2str(datevec(now));
datetime(ismember(datetime,' ')) = []; %remove spaces

%if no fileroot is given, use the current datetime
if( ~exist('fileroot') || isempty(fileroot) )
    fileroot = datetime;
% otherwise use current datetime and append fileroot
else
    fileroot = strcat(datetime, '_', fileroot);
end

%get handles
h = get(0,'children');
h = sort(h);    %sorts them according to figure number

if(length(h)~=0)
    %workaround: print to postscript first
    tempfile = ['test',num2str(rand),'.ps'];
    for i=1:length(h)
        print(h(i),'-dpsc2',tempfile,'-append')
    end

    %then convert to pdf
    filename = fullfile('Saves', [fileroot,'.pdf']);
    ps2pdf('psfile', tempfile, 'pdffile', filename, 'deletepsfile', 1)
end


