%Train an L1 defender using Jaakkola
function filename = TrainD1J3wayAndSave( ppack, filename )

if ( ~exist('ppack') )
    ppack.params = Set_params();
    ppack = Load_mats(ppack);
end
if ( ~exist('filename') )
    filename = [];
end

close all;

clear L0Attacker;

% Switch between personalities when he exists
L0Attacker.switchmode = [2, 3];
L0Attacker.switchthetaA = [0, 0.07];
L0Attacker.switchprobs = [0.8, 0.2];
%

% Modifying Scott's L0 Defender to be used as policy initialization in
% Jaakoola training.
ScottLODefinit= Scott_L0Def_forinit(ppack.params, ppack.BinsOrSet, ppack.CPTs);

fprintf('\nTrain L1 Defender...')
method = 2; % 1: InitL0-NoObsNoTrain, 2: InitRand-NoObsFillL0
tic
L1Defender = Train_defender_Jaakkola( L0Attacker, ppack.params, ppack.BinsOrSet, ppack.CPTs, ppack.ER, 1, ScottLODefinit, method);
toc

L0Attacker.mode = 2;
L0Attacker.thetaA = 0;
L0Attacker.exist = 0; % start off not exist
fprintf('\nRandom Attacker enters at time 45')
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack, 45 );

L0Attacker.mode = 3;
L0Attacker.thetaA = 0.07;
L0Attacker.exist = 0;  % start off not exist
fprintf('\nDrift-and-strike Attacker enters at time 45')
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack, 45 );

fprintf('\nSaving...');
filename = SaveDescription( SaveAllOpenPlots(filename), ppack );
save( fullfile('Saves',[filename '.mat']), 'L1Defender');

fprintf('\nTrainD1JandSave Done!\n\n');