%Train an L1 defender using Jaakkola
function [filename, SW, SW_SEM, AAR, AAR_SEM, DAR, DAR_SEM] = TrainD1JandSave_SW2( ppack, filename )
fprintf('\nD1vsA0 Social Welfare Study, continuous training version...')

if ( ~exist('ppack') )
    ppack.params = Set_params();
    ppack = Load_mats(ppack);
    ppack.useContL0 = 0;
end
if ( ~exist('filename') )
    filename = [];
end

close all;

clear L0Attacker;
L0Attacker.mode = 3;
L0Attacker.thetaA = 0.07;
%ppack.useContL0 = 0; %use this when you want to run this file alone (so you
                      %have to define the parameter here)
L0Attacker.useContL0 = ppack.useContL0; %set to 1 or 0 at GridOptimization_exh2.m

fprintf('\nTrain L1 Defender...')
%L1Defender = Train_defender_Jaakkola_c( L0Attacker, ppack, 1);

%Use the following block of code for HiFi training
ScottLODefinit= Scott_L0Def_forinit(ppack.params, ppack.BinsOrSet, ppack.CPTs);
method = 0;

%If Defender is desired to be initialized to a certain defender then use
%the code below
if method == 3 
    load('Saves_RealEnvTestJaakAndJamesQuestions/2012727175648_AvValueCheckForRealEnv', ...
        'L1Defender');
    ScottLODefinit = L1Defender;
end
clear L1Defender;

L1Defender = Train_defender_Jaakkola( L0Attacker, ppack, 1, ScottLODefinit, method);


fprintf('\nSimulate L1 Defender vs L0 Attacker in continuous (does not exist)...')
L0Attacker.exist = 0;
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

fprintf('\nSimulate L1 Defender vs L0 Attacker in continuous (exist)...')
L0Attacker.exist = 1;
[D1A0states, D1A0defRewards, D1A0attRewards] = Simulate ( L1Defender, L0Attacker, 1, ppack );

fprintf('Average reward in continuous...')
[defAvgReward, defAvgRewardSEM, attAvgReward, attAvgRewardSEM, ...
    defAvgNumMove, defAvgNumMoveSEM, AvgSocWelf, AvgSocWelfSEM] ... 
                = Average_rewards( L1Defender, L0Attacker, 100, false, ppack);
SW = AvgSocWelf;
SW_SEM = AvgSocWelfSEM;
AAR = attAvgReward;
AAR_SEM = attAvgRewardSEM;
DAR = defAvgReward;
DAR_SEM = defAvgRewardSEM;

fprintf('\nSaving...');
filename = SaveDescription( SaveAllOpenPlots(filename), ppack );
save( fullfile('Saves',[filename '.mat']), 'L1Defender','ppack', ...
    'defAvgReward','defAvgRewardSEM','attAvgReward', 'attAvgRewardSEM', ...
    'defAvgNumMove', 'defAvgNumMoveSEM', 'AvgSocWelf', 'AvgSocWelfSEM');

fprintf('\nTrainD1JandSave_SW2 Done!\n\n');