function [Attacker] = Train_attacker_FQI(Defender, ppack, makeplot)
%Learn a RL Attacker assuming given an Defender
%using Fitted Q-iteration algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Defender = defender
%Output: Attacker = learned RL attacker

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;

fprintf('Fitted Q iteration attacker...\n')

params.fqi = Set_params_FQI();
Attacker.trainParams = params.fqi;

%create a new attacker
clear Attacker;
Attacker.mode = 2;  %Attacker plays uniform random policy
trackIndex = 0; %for tracking

%Collect samples
clear samples;
sm = 1;
for ep = 1:params.fqi.nSamplesEpsDef
    % attacker always exists for attacker training
    Attacker.exist = 1;
    [states, ~, ~, ~, ~, ~, ~] = Simulate( Defender, Attacker, 0, ppack, inf );
    
    %Assemble into state transition samples
    for st = 1:length(states)-1
        samples{sm}.x = [states{st}{5}.V2; states{st}{5}.V3; ...
            states{st}{5}.q3_prev; states{st}{5}.SA]; %MA
        samples{sm}.u = states{st}{7}.q3; %A
        samples{sm}.xp = [states{st+1}{5}.V2; states{st+1}{5}.V3; ...
            states{st+1}{5}.q3_prev; states{st+1}{5}.SA]; %MA'
        samples{sm}.r = Attacker_reward(states{st}, ppack.params);
        sm = sm + 1;
    end
end

%Linear FQI
if ( params.fqi.mode == 1 )
    %Initial weights
    v = [samples{1}.x; samples{1}.u];
    f = FQI_GetFeatures(v);
    w0 = ones(length(f),1);

    wout = FitQ_linear( w0, samples, params.fqi.nItersAtt, params.fqi.gamma, BinsOrSet.q3Set );

    %Return final weights
    Attacker.mode = 5;
    Attacker.QWts = wout;

end

% if makeplot
%     figure
%     plot(Defender.InnerLoopRewAv);
%     title('Running window average reward');
%     
%     figure
%     plot(Defender.avreward);
%     title('Average reward');
% end


end
