function [Defender] = Train_defender_FQI(Attacker, ppack, makeplot)
%Learn a RL Defender assuming given an Attacker
%using Fitted Q-iteration algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Attacker = attacker
%Output: Defender = learned RL defender

params = ppack.params;
BinsOrSet = ppack.BinsOrSet;

fprintf('Fitted Q iteration defender...\n')

params.fqi = Set_params_FQI();
Defender.trainParams = params.fqi;

%create a new defender
clear Defender;
Defender.mode = 2;  %Defender plays uniform random policy


%Collect samples
clear samples;
sm = 1;
for ep = 1:params.fqi.nSamplesEpsDef
    % coin flipping on attacker's existence
    Attacker.exist = genAttExists(params.ProbAttExist);
    [states, ~, ~, ~, ~, ~, ~] = Simulate( Defender, Attacker, 0, ppack, inf );
    
    %Assemble into state transition samples
    for st = 1:length(states)-1
        samples{sm}.x = [states{st}{4}.V1; states{st}{4}.V2; states{st}{4}.V3; ...
            states{st}{4}.D_prev; states{st}{4}.SD]; %MD
        samples{sm}.u = states{st}{6}; %D
        samples{sm}.xp = [states{st+1}{4}.V1; states{st+1}{4}.V2; states{st+1}{4}.V3; ...
            states{st+1}{4}.D_prev; states{st+1}{4}.SD]; %MD'
        samples{sm}.r = Defender_reward(states{st}, ppack.params);
        sm = sm + 1;
    end
end

% Linear FQI
if ( params.fqi.mode == 1 )
    %Initial weights
    v = [samples{1}.x; samples{1}.u];
    f = FQI_GetFeatures(v);
    w0 = ones(length(f),1);
    
    wout = FitQ_linear( w0, samples, params.fqi.nItersDef, params.fqi.gamma, params.dXset );

    %Return final weights
    Defender.mode = 5;
    Defender.QWts = wout;
end

% if makeplot
%     figure
%     plot(Defender.InnerLoopRewAv);
%     title('Running window average reward');
%     
%     figure
%     plot(Defender.avreward);
%     title('Average reward');
% end


end
