function [Attacker] = Train_attacker_Jaakkola(Defender, params, BinsOrSet, CPTs, ER)
%Learn a RL Defender assuming given an Attacker
%using Jaakkola algorithm.
%This version uses epoch-wise updating of the policy.
%Input: Attacker = attacker
%Output: Defender = learned RL defender
%global params;

fprintf('Jaakkola_attacker...\n')

params.jaak = Set_params_Jaak();

%create a new defender
clear Attacker;
Attacker.mode = params.policy_mode;     %Put attacker into policy mode
Attacker.exist=1;

trackIndex = 0; %for tracking

%Initializing the Jaakkola matrices and the average reward
r_av = 0;
Attacker.K = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3);
Attacker.beta = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3);
Attacker.Q = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3);
Attacker.C = ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3);

% Attacker.K_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.beta_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.V = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);
% Attacker.C_msg = ones(BinsOrSet.N_V2,BinsOrSet.N_V3);

Attacker.gamma_Q=params.jaak.gamma; %ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3)*params.jaak.gamma;
Attacker.gamma_V=params.jaak.gamma; %ones(BinsOrSet.N_V2,BinsOrSet.N_V3)*params.jaak.gamma;


%Initializing the stochastic Defender Policy Defender.pi(a|m) and greedy
%policy pi1(a|m) (see the Jaakkola paper for explanations)
Attacker.pi=ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3)/(BinsOrSet.N_q3);
pi1=ones(BinsOrSet.N_V2,BinsOrSet.N_V3,BinsOrSet.N_q3)/(BinsOrSet.N_q3);


%Initializing tracking matrices

Attacker.deltaQ=ones(1,1000);
delta_old=1;

Attacker.avreward=zeros(1,2000);
Attacker.avreward_in=zeros(1,2000);

B = Sample_slice_d([], Defender, Attacker, CPTs, params, BinsOrSet);
ep=1;

Attacker.pi_old=Attacker.pi*2;

while (max(max(sum(Attacker.pi_old.*log(Attacker.pi_old./Attacker.pi),3)))*10^7>5 ...
    && max(max(-sum(Attacker.pi.*log2(Attacker.pi),3)))>0.8);   %min(Attacker.pi(:))>0.03
%while ep<2
    ep
    
    kl=max(max(sum(Attacker.pi_old.*log(Attacker.pi_old./Attacker.pi),3)))*10^7
    ent=max(max(-sum(Attacker.pi.*log2(Attacker.pi),3)))
    
    Q_old = Attacker.Q;   %This will be used for Delta_Q computation
    Attacker.pi_old = Attacker.pi; %This will be used for Delta_pi computation
    
    %B = Sample_slice_d([], Defender, Attacker,params);
    
    
    for t = 1:50 %params.jaak.nSimSteps
        Q_old_in = Attacker.Q;   %This will be used for Delta_Q computation
        pi_old_in = Attacker.pi; %This will be used for Delta_pi computation
        %Extract variables at t
%         [S_t,OD_t,MD_t,dX_t,OA_t,MA_t,m_t,L_t] = Slice_unpack_d(B);
        
        [S_t,MD_t,MA_t,dX_t,A_t,L_t] = Slice_unpack_d(B);
        
        V1_t = S_t.V1index;
        V2_t = S_t.V2index;
        V3_t = S_t.V3index;
        a_t = A_t.q3index; %a_t is the index of q3
        
        %calculate the reward
        
        r = ER.Att(V2_t);
        
        
        %Updating Jaakkola message-action matrices M(m,a) for m_t=m, a_t=a
%         Defender.K(OD_t,MD_t,a_t)=Defender.K(OD_t,MD_t,a_t) + 1;
        Attacker.K(V2_t,V3_t,a_t)=Attacker.K(V2_t,V3_t,a_t) + 1;    
        
        Attacker.beta=Attacker.gamma_Q*Attacker.beta;
        Attacker.Q=Attacker.Q+Attacker.beta*(r-r_av);
        
        Attacker.Q(V2_t,V3_t,a_t) = Attacker.Q(V2_t,V3_t,a_t) - Attacker.beta(V2_t,V3_t,a_t)*(r-r_av);
        Attacker.beta(V2_t,V3_t,a_t)=(1-(1/Attacker.K(V2_t,V3_t,a_t)))*Attacker.beta(V2_t,V3_t,a_t)+1/Attacker.K(V2_t,V3_t,a_t);
        Attacker.Q(V2_t,V3_t,a_t)=(1-(1/Attacker.K(V2_t,V3_t,a_t)))*Attacker.Q(V2_t,V3_t,a_t)+Attacker.beta(V2_t,V3_t,a_t)*(r-r_av);  

        
        %Update the average reward estimate with the current reward
        r_av_new = r_av + (r-r_av)/t;
        
        %Transform Jaakkola message-action matrices M(m,a) for m_t=m,
        %a_t=a using the new average reward estimate

        
        Attacker.C = Attacker.C + Attacker.beta;
        Attacker.Q = Attacker.Q - Attacker.C*(r_av_new-r_av);
        
        Attacker.Q(V2_t,V3_t,a_t) = Attacker.Q(V2_t,V3_t,a_t) + Attacker.C(V2_t,V3_t,a_t)*(r_av_new-r_av);      
        Attacker.C(V2_t,V3_t,a_t) = Attacker.C(V2_t,V3_t,a_t) - Attacker.beta(V2_t,V3_t,a_t);
        
        Attacker.C(V2_t,V3_t,a_t) = (1-(1/Attacker.K(V2_t,V3_t,a_t)))*Attacker.C(V2_t,V3_t,a_t)+Attacker.beta(V2_t,V3_t,a_t);
        Attacker.Q(V2_t,V3_t,a_t) = Attacker.Q(V2_t,V3_t,a_t)-Attacker.C(V2_t,V3_t,a_t)*(r_av_new-r_av);
        
        r_av = r_av_new;
        
        %Sample the next step
        B = Sample_slice_d(B, Defender, Attacker, CPTs, params, BinsOrSet);
        
        %Extract variables at t+1
        [S_tp1,MD_t,MA_t,dX_tp1,A_tp1,L_tp1] = Slice_unpack_d(B);
        
        V1_tp1 = S_tp1.V1index;
        V2_tp1 = S_tp1.V2index;
        V3_tp1 = S_tp1.V3index;
        a_tp1 = A_tp1.q3index; %a_tp1 is the index of q3_tp1
        
         
        %For tracking
        trackIndex = trackIndex + 1;
               
          delta=sum(sum(sum(sum(abs(Attacker.Q)-abs(Q_old_in)))));
          Attacker.deltaQ(t) = delta_old-delta;
          delta_old=delta;



          Attacker.avreward_in(t)=r_av;
        
    end
    
    %Update the Attacker Policy Attacker.pi(a|m) at the end of the episode
    %by determining the greedy policy pi1 and then updating the Attacker
    %policy towards the greedy policy
    
    [~,ind]=max(Attacker.Q,[],3);
    for j=1:BinsOrSet.N_V2
        for k=1:BinsOrSet.N_V3
            for z=1:BinsOrSet.N_q3
                if z==ind(j,k)
                    Attacker.pi(j,k,z)=(1-params.jaak.epsln)*Attacker.pi(j,k,z)+params.jaak.epsln*1;
                else
                    Attacker.pi(j,k,z)=(1-params.jaak.epsln)*Attacker.pi(j,k,z)+params.jaak.epsln*0;
                end
            end
            Attacker.pi(j,k,1:BinsOrSet.N_q3)=Attacker.pi(j,k,1:BinsOrSet.N_q3)/sum(Attacker.pi(j,k,1:BinsOrSet.N_q3));
        end
    end
    
    
    
    
    
    %For tracking
    Attacker.avreward(ep)=r_av;
       
    ep=ep+1;
end


end
