function [Attacker] = Train_attacker_SARSANN( Defender, params )
%Learn an RL Defender assuming given an Defender
%using SARSA and a Neural Network approximator.
%This version uses epoch-wise updating.
%Input: Defender = defender
%Output: Attacker = learned RL attacker
%global params;

fprintf('Sarsa_attacker_NN...\n')

params.sarsann = Set_params_SARSANN();

%create a new attacker
clear Attacker;
Attacker.mode = params.policy_mode;     %put attacker into policy mode

%Initialize neural net
N_neurons = 50; %number of neurons in the hidden layer of the NN
scale = 1.0;  %scales the initial random weights
bias = 2.0;     %biases the initial random weights
%Declare a new feedforward (multi-layer perceptron) NN
Attacker.thetaA = newff( [0,2; 0,2; -2,2; -2,2; -1,1; -10,10;
        -10,10], [0 0.5], [N_neurons], {'tansig' 'purelin'});
Attacker.thetaA.IW{1} = scale*(rand(N_neurons,7))+bias;
Attacker.thetaA.LW{2,1} = scale*(rand(1,N_neurons))+bias;
Attacker.thetaA.b{1} = scale*(rand(N_neurons,1))+bias;
Attacker.thetaA.b{2} = scale*(rand(1,1))+bias;
%configure the regulator
Attacker.thetaA.divideFcn = '';
Attacker.thetaA.performFcn = 'msereg';
Attacker.thetaA.performParam.ratio = 0.1;
%configure learning rate
Attacker.thetaA.inputWeights{1}.learnParam.lr = params.alpha;
Attacker.thetaA.layerWeights{2,1}.learnParam.lr = params.alpha;
Attacker.thetaA.biases{1}.learnParam.lr = params.alpha;
Attacker.thetaA.biases{2}.learnParam.lr = params.alpha;

%Initialize tracking variables
Attacker.trainReward = zeros(params.nTrainEpsAtt,params.nSimSteps);
Attacker.trainIO = zeros(9, params.nTrainEpsAtt*params.nSimSteps);
Attacker.trainErr = zeros(params.nTrainEpsAtt,params.nSimSteps);
Attacker.trainPerf = zeros(1, params.nTrainEpsAtt);
Attacker.trainLW = zeros(N_neurons,params.nTrainEpsAtt);

%Parameter scheduling
%No scheduling = constant at the desired eps
if ( params.schedule_mode == 0 )
    schedule = params.epsilonA * ones(1,params.nTrainEpsAtt);
%3-segment scheduling
elseif( params.schedule_mode == 2)
   %eps-greedy case
   if( params.policy_mode == 7 )
       schedule = Schedule_3_seg(params.nTrainEpsAtt,params.schedule_start,params.epsilonA);
   else
       %error
       assert(false);
   end
else
    %error
    assert(false);
end

trackIndex = 0;     %for tracking
for ep = 1:params.nTrainEpsAtt
   ep
   
   %a place to collect all the NN training data
   inputs = zeros(7,params.nSimSteps);
   Qtargets = zeros(1,params.nSimSteps);
   
   %deal with epsilon scheduling
   Attacker.eps = schedule(ep);
   schedule(ep)
   B = Sample_slice([], Defender, Attacker,params);
   
   for t = 1:params.nSimSteps
       %Extract variables at t
       [S_t,OD_t,MD_t,dX_t,OA_t,MA_t,m_t,L_t] = Slice_unpack(B);
       a_t = m_t;
       phi_t = [OA_t; MA_t(1:2)]; %"s" = [Observation; Memory]
       Qsa_t = sim(Attacker.thetaA, [phi_t; a_t]);   %simulate neural net
       
       %Sample the next step
       B = Sample_slice(B, Defender, Attacker,params);
       
       %Extract variables at t+1
       [S_tp1,OD_tp1,MD_tp1,dX_tp1,OA_tp1,MA_tp1,m_tp1,L_tp1] = Slice_unpack(B);
       a_tp1 = m_tp1;
       phi_tp1 = [OA_tp1; MA_tp1(1:2)]; %"s" = [Observation; Memory(1)]
       Qsa_tp1 = sim(Attacker.thetaA, [phi_tp1; a_tp1]);   %simulate neural net
       r_tp1 = Attacker_reward(B,params);
       
       %SARSA
       if ( params.learn_mode == 1 )
           Qtarget = r_tp1 + params.gamma*Qsa_tp1;  %desired Q-value
       %Q-learning
       elseif ( params.learn_mode == 2 )
           Qtarget = r_tp1 + params.gamma*max_Q(Attacker.thetaA,phi_tp1,params);
       else
           assert(false);   %error
       end
       
       inputs(:,t) = [phi_t; a_t];
       Qtargets(t) = Qtarget;
       
       %For tracking
       trackIndex = trackIndex + 1;
       Attacker.trainIO(:,trackIndex) = [phi_t; a_t; Qsa_t; Qtarget];
       Attacker.trainReward(ep,t) = r_tp1;
       
   end
   
   %train NN at the end of the episode
   Attacker.thetaA.adaptParam.passes = 20;      %adapt faster by making multiple passes
   r = randperm(params.nSimSteps);  %randomize the permutations to learn better
   [Attacker.thetaA,Y,E,Pf,Af,tr] = adapt(Attacker.thetaA,inputs(:,r),Qtargets(:,r));  %adapt NN

   %For tracking
   Attacker.trainErr(ep,:) = E;
   Attacker.trainPerf(ep) = tr.perf;
   Attacker.trainLW(:,ep) = Attacker.thetaA.LW{2,1}';
   assert(max(Attacker.thetaA.LW{2,1}) < 1e10 );    %make sure the weights aren't exploding
   
   %display training progress/the mean reward this episode
   mean_reward = mean(Attacker.trainReward(ep,:))
   total_error = E*E'
end

end

function Qsa_max = max_Q( net, phi,params )
%Returns the max of Qsa over possible actions
%used in Q-learning
%global params;

Qsa = zeros(2*params.M+1,1);
for a_tp1 = -params.M:params.M
    index = a_tp1 + params.M + 1;       %Convert to an index
    Qsa(index) = sim( net, [phi; a_tp1] );
end
Qsa;
Qsa_max = max(Qsa);

end
