function wout = FitQ_linear( w0, samples, nIters, gamma, U, makeplot )
%wout = FitQ_linear( w0, samples, nIters, gamma, U )
%initial weights w0
%state transitions samples
%number of iterations nIters
%discount factor gamma
%vector of all possible (discrete) actions U

w = w0;

% Prep the input matrix
v = [samples{1}.x; samples{1}.u];
f = FQI_GetFeatures(v);
inputs = zeros(length(samples),length(f));

for sm = 1:length(samples)
    %Assemble input matrix from x, u for each sample
    cur = samples{sm};
    v = [cur.x; cur.u];
    %[FQI_GetFeatures(v)]'
    inputs(sm,:) = [FQI_GetFeatures(v)]';
end

for ni = 1:nIters
    %Prep the target matrix
    Qtargets = zeros(length(samples),1);
    for sm = 1:length(samples)        
        %Figure out Qtargets from x', u', r, W for each sample
        cur = samples{sm};
        Qtargets(sm) = cur.r + gamma*FitQLin_GetQmax(w, cur.xp, U);
    end
    
    %Linear least squares fit
    %w = inputs\Qtargets
    %[w,status] = l1_ls(inputs,Qtargets,0.5);
    w = pinv(inputs)*Qtargets;
    wtrack{ni} = w;
end

wout = w;

if ( exist('makeplot') && makeplot )
    figure;
    %cell2mat(wtrack)
    plot( 1:nIters, cell2mat(wtrack)' );
    xlabel('Iteration');
    ylabel('Weights');
    title('Learning Weights');
end

end

