function [player, info] = Train_FQI(SIM, playerList, playerNumber, makeplot)
%Learn a RL player assuming given other players
%using Fitted Q-iteration algorithm.
%This version uses epoch-wise updating of the policy.
%Input: SIM = simulation object
%       playerList = vector of player objects
%       playerNumber = the position in the playerList that you want to
%       train
%Output: player = learned RL player

fprintf(strcat('Fitted Q iteration: Player ', num2str(playerNumber), '...'));

params.fqi = Set_params_FQI();
info.trainParams = params.fqi;

%Default makeplot to true
if(~exist('makeplot'))
    makeplot = true;
end

%create a new player
props.moveSet = playerList(playerNumber).GetMoveSet();
playerList(playerNumber) = playerList(playerNumber).SetMode( 2, props );

%Collect samples
samples = cell(params.fqi.nSamplesEps*SIM.GetNumSteps(),1);
sm = 1;
SIM = SIM.SetPlayerList( playerList );
for ep = 1:params.fqi.nSamplesEps
    % coin flipping on attacker's existence
    %Attacker.exist = genAttExists(params.ProbAttExist);
    [trackPGS, trackMems, trackMoves] = SIM.Run();
    
    %Assemble into state transition samples
    for st = 1:SIM.GetNumSteps()
        samples{sm}.x = [trackMems{st}{playerNumber}.vec]; %Mem
        samples{sm}.u = trackMoves{st}{playerNumber}; %Move
        samples{sm}.xp = [trackMems{st+1}{playerNumber}.vec]; %Mem'
        samples{sm}.r = playerList(playerNumber).GetReward( trackPGS{st}, ... 
            trackMems{st}, trackMoves{st}, SIM.params ); %Reward
        sm = sm + 1;
    end
end

% Linear basis functions FQI
if ( params.fqi.method == 1 )
    fprintf('\nLinear regression');
    %Initial weights
    v = [samples{1}.x; samples{1}.u];
    f = FQI_GetFeatures(v);
    w0 = ones(length(f),1);
    
    wout = FitQ_linear( w0, samples, params.fqi.nIters, params.fqi.gamma,... 
            playerList(playerNumber).GetMoveSet(), makeplot );

    %Return final weights
    clear props;
    props.QWts = wout;
    player = playerList(playerNumber).SetMode( 5, props );

% Neural fitted Q-iteration
elseif ( params.fqi.method == 2 )
    fprintf('\nNeural network regression');
    %Initialize NN
    v = [samples{1}.x; samples{1}.u];
    NumNeurons = 20;
    net = newff(rand(length(v),length(samples)-1), ...
            rand(1,length(samples)-1), NumNeurons);
    net = FitQ_NN(net, samples, params.fqi.nIters, params.fqi.gamma,... 
            playerList(playerNumber).GetMoveSet(), makeplot );
        
    %Return final weights
    clear props;
    player = playerList(playerNumber);
    props.net = net;
    player = playerList(playerNumber).SetMode( 6, props );
end


end
