
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20101025	Created
//	rlee		20110816	Changed to a abstract parent class
//
// *************************************************************************

#ifndef INCLUDED_CPD_H
#define INCLUDED_CPD_H

#include "helper.h"
#include "types.h"

using namespace std;

class cpd
{
public:

	/**	Constructor */
	cpd( void ){};

	/**	Destructor */
	virtual ~ cpd(){};

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vectors
	 *	returns: sample value of node
	 */
	virtual nfg_Vec		Sample_node ( const nfg_Vecs& paVecs ) 
	{ 
		bassert(false);
		return nfg_Vec(); 
	};
	/**
	 * Get_probability: Get probability of a vector given parents
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	virtual double		Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs ) 
	{ 
		bassert(false);		// Should not 
		return 0; 
	};
	/**
	 * Get_probability_prop: Get probability of a vector given parents, but only guarantees
	 *		proportionality.  e.g. for contiuous pdfs
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	virtual double		Get_probability_prop ( const nfg_Vec& outVec, 
												const nfg_Vecs& paVecs )
	{
		// Try calling the exact version as default
		return Get_probability ( outVec, paVecs );
	};
	/**
	 * Get_util: Get utility value (from a player)
	 *	inst - an instantiation of the net
	 *	returns: utility value (double)
	 */
	virtual double		Get_util ( const nfg_Inst& inst ) 
	{ 
		bassert(false);		// fail
		return 0; 
	};


private:

	/// Variables

	/// Methods


};

#endif

