
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120913	Created
//
// *************************************************************************

#include "cpd_constant.h"

using namespace nfg;

// *************************************************************************
// *************************************************************************
cpd_Constant::cpd_Constant( const double constVal )
{
	m_constVal = constVal;
}
// *************************************************************************
// *************************************************************************
cpd_Constant::~cpd_Constant ( void )
{
}
// *************************************************************************
// *************************************************************************
nfg_Vec cpd_Constant::Sample_node ( const nfg_Vecs& paVecs )
{
	// ignore parents and set to constant

	return nfg_Vec(1, m_constVal);
}
// *************************************************************************
// *************************************************************************
double cpd_Constant::Get_probability( const nfg_Vec& outVec, const nfg_Vecs& paVecs )
{
	// ignore parents and check for constant value

	return (outVec.at(0)==m_constVal) ? 1.0 : 0.0;
}
// *************************************************************************
// *************************************************************************
void cpd_Constant::Unit_test ( void )
{
	//	Set up variance vector
	cpd_Constant con( 1.5 );

	//	Set up parents vector list
	nfg_Vecs paVecs;

	//	Test sample_node()
	nfg_Vec outVec = con.Sample_node( paVecs );
	Print_vec(outVec);

	//	Test Get_prob()
	cout << "Prob: " << con.Get_probability_prop( outVec,  paVecs ) << endl;

	outVec.at(0) = 0.0;
	cout << "Prob: " << con.Get_probability_prop( outVec,  paVecs ) << endl;

	cout << "Unit test completed." << endl << endl;
}

