
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110328	Created
//	rlee		20110816	Converted to inherited class
//
// *************************************************************************

#ifndef INCLUDED_CPD_DECISION_H
#define INCLUDED_CPD_DECISION_H


#include <boost/function.hpp>
#include "types.h"
#include "helper.h"
#include "cpd.h"
#include "player.h"

using namespace std;
using namespace boost;

//	Decision node
class cpd_Decision : public cpd
{
public:

	/**	Constructor */
	cpd_Decision( Player& player );

	/**	Destructor */
	virtual ~ cpd_Decision();

	/// Variables

	///	Methods
	/**
	 * Get_util: Get utility value (from a player)
	 *	inst - an instantiation of the net
	 *	returns: utility value (double)
	 */
	double	Get_util ( const nfg_Inst& instVecs );
	/**
	 * Get_probability: Get probability of a vector given parents (
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	double	Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );
	

private:

	/// Variables
	nfg_UtilFunc			m_fo_Get_util;			// call back function for providing utility function

	/// Methods

};

#endif

