
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120914	Created
//
// *************************************************************************

#ifndef INCLUDED_CPD_GLUE_H
#define INCLUDED_CPD_GLUE_H

#include "types.h"
#include "helper.h"
#include "cpd.h"

/**
 * cpd_Glue: CPD class that implements a simple adder.
 */
class cpd_Glue : public cpd
{
public:

	/**	Constructor */
	cpd_Glue( void );

	/**	Destructor */
	virtual ~ cpd_Glue();

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vectors
	 *	returns: sample value of node
	 */
	nfg_Vec	Sample_node ( const nfg_Vecs& paVecs );
	/**
	 * Get_probability: Get probability of a vector given parents
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	double	Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );

	// Debug
	/**
	 * Unit_test: test to verify that this cpd works correctly.
	 */
	static void Unit_test ( void );

private:

	/// Variables

	/// Methods

};

#endif

