
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20101025	Created
//	rlee		20110816	Converted to inherited class
//
// *************************************************************************

#ifndef INCLUDED_CPD_REROUTE_H
#define INCLUDED_CPD_REROUTE_H

#include "types.h"
#include "helper.h"
#include "cpd.h"

/**
 * cpd_Reroute: CPD class that implements routing and subsets
 *		of vector components from parent to current node.
 *		Expects a single parent input vector.
 *		e.g. paVec=[2 4 3 2 5], set paIndex=[0 0 4], then outVec=[2 2 5].
 */
class cpd_Reroute : public cpd
{
public:

	/**	Constructor */
	/**
	 * cpd_Reroute: Construct a routing node
	 *	paIndex - a vector containing indices into paVec
	 */
	cpd_Reroute( const nfg_Dim& paIndex );

	/**	Destructor */
	virtual ~ cpd_Reroute();

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vector (expects single parent vec)
	 *	returns: sample value of node
	 */
	nfg_Vec	Sample_node ( const nfg_Vecs& paVecs );
	/**
	 * Get_probability: Get probability of a vector given parents, but only guarantees
	 *		proportionality.  e.g. for contiuous pdfs
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vector
	 *	returns: probability that this occurred
	 */
	double	Get_probability ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );

	// Debug
	/**
	 * Unit_test: test to verify that this cpd works correctly.
	 */
	static void Unit_test ( void );

private:

	/// Variables
	nfg_Dim			m_paIndex;		// vector containing indices into paVec to route to outVec

	/// Methods

};

#endif

