
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20101025	Created
//	rlee		20110816	Converted to inherited class
//
// *************************************************************************

#ifndef INCLUDED_CPD_UNIFORM_H
#define INCLUDED_CPD_UNIFORM_H

#include "types.h"
#include "helper.h"
#include "cpd.h"

class cpd_Uniform : public cpd
{
public:

	struct Range
	{
		Range( double min, double max )
		{
			m_min = min;
			m_max = max;
		};

		double	m_min;
		double	m_max;
	};

	/**	Constructor */
	/**
	 * cpd_Uniform: Create a cpd object for continuous uniform distribution, not based
	 *		on parents.
	 *	range - vector of min/max values, one pair for each dimension
	 */
	cpd_Uniform( const vector<Range>& range );
	/**
	 * cpd_Uniform: Create a cpd object for continuous uniform distribution, not based
	 *		on parents.
	 *	rangeMinVec - vector of range min values, one for each dimension
	 *	rangeMaxVec - vector of range max values, one for each dimension
	 */
	cpd_Uniform( const nfg_Vec& rangeMinVec, const nfg_Vec& rangeMaxVec );

	/**	Destructor */
	virtual ~ cpd_Uniform();

	/// Variables

	///	Methods
	/**
	 * Sample_node: Sample the node given parents
	 *	paVecs - parent vectors
	 *	returns: sample value of node
	 */
	nfg_Vec	Sample_node ( const nfg_Vecs& paVecs );
	/**
	 * Get_probability_prop: Get probability of a vector given parents, but only guarantees
	 *		proportionality.  e.g. for contiuous pdfs
	 *	outVec - a sample/realization of the node
	 *	paVecs - parent vectors
	 *	returns: probability that this occurred
	 */
	double	Get_probability_prop ( const nfg_Vec& outVec, const nfg_Vecs& paVecs );

	//	Debug
	static void Unit_test( void );

private:

	/// Variables
	vector<Range>	m_range;

	/// Methods
	
};

#endif

