
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110401	Created
//
// *************************************************************************

#ifndef INCLUDED_IF_BAYESNETTOOLBOX_H
#define INCLUDED_IF_BAYESNETTOOLBOX_H

#include "types.h"
#include "helper.h"
#include "bayesnet.h"
#include "ibayesnet.h"
#include "if_matlab.h"

//	Interface to Kevin Murphy's Bayes Net Toolbox
//	BNT is in Matlab
class if_BayesNetToolbox : public if_Matlab
{
public:

	/**	Constructor */
	if_BayesNetToolbox( void );

	/**	Destructor */
	virtual ~ if_BayesNetToolbox();

	/// Variables

	///	Methods
	/**
	 * Draw_graph: Outputs a Matlab m-file that automatically draws a dag to a figure.
	 *		Requires BNT to be installed in Matlab.
	 *	filename - output filename
	 *	bnet - nfg object
	 *	append - true opens a file and appends the draw graph command to the end of the script
	 *		false will overwrite the file.
	 *	dagName - to overwrite the variable name storing the dag.
	 *	labelsName - to overwrite the variable name that stores all the label information.
	 */
	//	Outputs an m-file that automatically draws a dag
	static void	Draw_graph( const string filename, BayesNet& bnet, bool append = false, 
								const string dagName = "dag", const string labelsName = "labels" );
	/**
	 * Draw_graph: Outputs a Matlab m-file that automatically draws a dag to a figure.
	 *		Requires BNT to be installed in Matlab.
	 *	filename - output filename
	 *	ibnet - infg object
	 *	append - true opens a file and appends the draw graph command to the end of the script
	 *		false will overwrite the file.
	 *	dagName - to overwrite the variable name storing the dag.
	 *	labelsName - to overwrite the variable name that stores all the label information.
	 */
	//	Outputs an m-file that automatically draws a dag
	static void	Draw_graph( const string filename, IBayesNet& ibnet, bool append = false, 
								const string dagName = "dag", const string labelsName = "labels" );


private:

	/// Variables


	/// Methods

};

#endif

