
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120914	Created
//
// *************************************************************************

#ifndef INCLUDED_IPLAYER_H
#define INCLUDED_IPLAYER_H

#include <list>
#include <boost/function.hpp>
#include "types.h"
#include "helper.h"

using namespace std;
using namespace boost;

/**
 * IPlayer class. Player that control
 *	Has 2 utility functions.
 */
class IPlayer
{
public:

	/**	Constructor */
	/**
	 * IPlayer: constructor.
	 *	fo_Get_util_base - function object of utility function for base bnet
	 *	fo_Get_util_kernel - function object of utility function for kernel bnet
	 *	name - optional - name of player
	 */
	IPlayer( const nfg_UtilFunc& fo_Get_util_base, const nfg_UtilFunc& fo_Get_util_kernel, 
					const string name = "" );

	/**	Destructor */
	virtual ~ IPlayer();

	/// Variables

	///	Methods
	/**
	 * Get_fo_of_Get_util_base: Get the function object leading to Get_util()
	 *	returns: function object of Get_util
	 */
	nfg_UtilFunc& Get_fo_of_Get_util_base ( void );
	/**
	 * Get_fo_of_Get_util_kernel: Get the function object leading to Get_util()
	 *	returns: function object of Get_util
	 */
	nfg_UtilFunc& Get_fo_of_Get_util_kernel ( void );
	/**
	 * Get_name: Get the player's name
	 *	returns: player name
	 */
	string Get_name ( void ) { return m_name; };
	/**
	 * Set_policy: Set the player's policy
	 *	fo_policy - function object to the policy
	 */
	void	Set_policy ( nfg_SampleFunc fo_policy );

	// Debug
	static void Unit_test ( void );		// test various controlled node list functionality

private:

	/// Variables
	nfg_UtilFunc			m_fo_Get_util_base;			// call back function for providing utility function
	nfg_UtilFunc			m_fo_Get_util_kernel;		// call back function for providing utility function
	nfg_SampleFunc			m_fo_policy;				// call back function to player's policy
	string					m_name;

	/// Methods

};

#endif

