
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20120710	Created
//
// *************************************************************************

#ifndef INCLUDED_IMERGEANDSPACE_H
#define INCLUDED_IMERGEANDSPACE_H

#include "helper.h"
#include "types.h"
#include "ibayesnet.h"

class net_IMergeAndSpace
{
public:

	// Make convenient labels for the vertices
	//	There's only one set because both base and kernel are the same
	enum { T1, S1, S2, D1, D2, T2, S3, S4, N };

	/**
	 * Run: Execute example sequence
	 */
	static void			Run ( void );

	/**
	 * CPD_S_Sample_node: Resultant position
	 *		of aircraft.
	 */
	class cpd_S : public cpd
	{
	public:
		nfg_Vec		Sample_node ( const nfg_Vecs& paVecs );
	};

	/** 
	 * Util_function_P1: P1's utility function
	 *	instVecs: an instantiation of the net
	 *	returns: utility value
	 */
	static double		Util_function_P1 ( const nfg_Inst& instVecs );
	/** 
	 * Util_function_P2: P2's utility function
	 *	instVecs: an instantiation of the net
	 *	returns: utility value
	 */
	static double		Util_function_P2 ( const nfg_Inst& instVecs );
	/**
	 * Sample_level0: Samples the player's level 0 policy. Plays a constant move.
	 *	paVecs - parent vectors, but are ignored
	 */
	static nfg_Vec		Sample_level0 ( const nfg_Vecs& paVecs );
	/**
	 * Analysis1: Performs a series of analyses based on a ibnet
	 *	ibnet - infg object
	 */
	static void			Analysis1 ( IBayesNet& ibnet );

private:

	/// Variables

	/// Methods

};

#endif

