
// *************************************************************************
//	LibNFG: Net-form game library
//
//	Author:		Ritchie Lee
//
//  Copyright 2011 United States Government as represented by the Administrator 
//  of the National Aeronautics and Space Administration.  All Rights Reserved.  
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not 
//  use this file except in compliance with the License.  You may obtain a copy 
//  of the License at http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software 
//  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
//  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the 
//  License for the specific language governing permissions and limitations 
//  under the License.
//
//	Revision Log
//
//	Username	Date		Description
//	-------------------------------------------------------
//	rlee		20110328	Created
//
// *************************************************************************

#ifndef INCLUDED_PLAYER_H
#define INCLUDED_PLAYER_H

#include <list>
#include <boost/function.hpp>
#include "types.h"
#include "helper.h"

using namespace std;
using namespace boost;

/**
 * Player class. One player may control multiple nodes.
 *	Has a utility function.
 */
class Player
{
public:

	/**	Constructor */
	/**
	 * Player: constructor.
	 *	fo_Get_util - function object of utility function
	 */
	Player( const nfg_UtilFunc& fo_Get_util, const string name = "" );

	/**	Destructor */
	virtual ~ Player();

	/// Variables

	///	Methods
	/**
	 * Get_fo_of_Get_util: Get the function object leading to Get_util()
	 *	returns: function object of Get_util
	 */
	nfg_UtilFunc& Get_fo_of_Get_util ( void );
	/**
	 * Add_controlled_node: Add a node to the list of player controlled nodes
	 *	nodeId - id of vertex to be added
	 */
	void Add_controlled_node ( const int nodeId );
	/**
	 * Is_controlled: Tests whether a node is controlled by the player
	 *	nodeId - id of vertex of interest
	 *	returns: true if controlled, false otherwise
	 */
	bool Is_controlled ( const int nodeId );
	/**
	 * Get_num_controlled_nodes: Get number of nodes controlled by player
	 *	returns: number of nodes controlled by player
	 */
	int Get_num_controlled_nodes ( void ) { return m_controlledNodes.size(); };
	/**
	 * Clear_controlled_nodes: Clear list of nodes controlled by player
	 */
	void Clear_controlled_nodes ( void ) { m_controlledNodes.clear(); };
	/**
	 * Get_name: Get the player's name
	 *	returns: player name
	 */
	string Get_name ( void ) { return m_name; };

	// Debug
	static void Unit_test ( void );		// test various controlled node list functionality

private:

	/// Variables
	nfg_UtilFunc			m_fo_Get_util;			// call back function for providing utility function
	list<int>				m_controlledNodes;		// a list of nodes that are controlled by the player
	string					m_name;

	/// Methods

};

#endif

