# -*- coding: utf-8 -*-
"""
Created on Wed Oct 31 10:15:43 2012

@author: Yildiray Yildiz, the main code is taken from 
http://nullege.com/codes/show/src%40p%40y%40pymc-HEAD%40pymc%40CommonDeterministics.py/
28/PyMCObjects.Deterministic/python

A proof of concept script creating a deterministic chance node 
object for the purpose of converting NFGs into PyMC models after supplying return 
functions to decision nodes in an outer loop
"""

import pymc as pm
import inspect
from types import *

class DetChanceNode(object):
    """
    L = DetChanceNode(name, lambda p1=p1, p2=p2: f(p1, p2)[,
        doc, dtype=None, trace=True, cache_depth=2, plot=None])
  
    Converts second argument, an anonymous function, into a
    Deterministic object with specified name.
  
    :Parameters:
      name : string
        The name of the deteriministic object to be created.
      lambda : function
        The function from which the deterministic object should
        be created. All arguments must be given default values!
      p1, p2, ... : any
        The parameters of lambda.
      other parameters :
        See docstring of Deterministic.
  
    :Note:
      Will work even if argument 'lambda' is a named function
      (defined using def)
  
    """
    def __init__(self, name, space = [], description="", *args, **kwds):
        # Checking types of the inputs
        # name is a string
        assert type(name) is StringType, "name is not a string: %r" % name             
        self.name = name
        # space is a list
        assert type(space) is ListType, "space is not a list, it is %r" % type(space)
        self.space = space
        # description is a string
        assert type(description) is StringType, "description is not a string:"+" %r" % description
        self.description = description
        
        self.eval_func = None
            
    def convert_to_pymc( self ):
        # extracting the parent names and values from the function definition
        (parent_names, junk0, junk1, parent_values) = inspect.getargspec(self.eval_func)
        
        # making sure that the function do not have * and ** arguments and arguments have
        # default values. Default values are needed because they are the parent assignments
        if junk0 is not None \
            or junk1 is not None \
            or parent_values is None:
            raise ValueError, '%s: All arguments to lam_fun must have default values.' % name
        if not len(parent_names) == len(parent_values):
            raise ValueError, '%s: All arguments to lam_fun must have default values.' % name
  
        # forming a dictionary with elements 'parent names':parent values
        parents_dict = dict(zip(parent_names[-len(parent_values):], parent_values))
        
        # Finally defining the pymc object
        self.pymc = pm.Deterministic(eval=self.eval_func, name=self.name, 
                                     parents=parents_dict, 
                                     doc=self.description, dtype=None, 
                                     trace=True, cache_depth=2, plot=None)

#$$ Example Usage
#$$ 

# Assume the parents of the deterministic node "final_pos" are "in_stk_pos", "d1" and "d2".
# Also assume that "final_pos" outputs the sum of its parent nodes.
# Following code explains how to create the pymc deterministic object "final_pos":
#
#def sum_parents( initial=in_stk_pos, d1move = d1, d2move = d2 ):
#     return initial + d1move - d2move
# 
#c = DetChanceNode( name='c', description='child node of node a and b' ) #parents will be captured from the eval_func
# 
#c.eval_func = sum_parents
#c.convert_to_pymc()